function [u2,mtf50,Qe,r,lsf,f,mtfSys] = DOFMTF50(fn,N,u,plotFlag,u2,QF,Qeth,MTFth,n,pitch,ff,lambda)
    %Calculates monochromatic 'sharpness' performance mtf50 and Edge Acutance
    %Qe for a lens with nominal focal length fn, f-number N, in-focus
    %distance u (m) - at distances u2 (m); set plotFlag to true for a plot, 
    %'acceptably in-focus' thresholds for Acutance Qe and MTF50 (lp/mm);
    %n is the power of 2 number of pix in LSF axis (output r in um),
    %sensor pitch (um), linear fill factor ff and wavelength lambda (um).
    %Assumes* W040 = QF/Nz^2*lambda and 'best focus' with W020=-W040.
    %mtf50 and Spatial freq f in lp/um at each u2, mtfSys the MTF curve. 
    %EXAMPLE: [u2,mtf50] = DOFMTF50(105,2,1.35,1,1.33:.001:1.37);
    %For narrow DOF ensure there are enough sample distances u2 near the peak.
    %By Jack Hogan  - Version 0.34 - Refer to:
    %https://www.strollswithmydog.com/dof-diffraction-setup/
    
    if nargin < 12                              %Defaults:
        lambda = 0.535;                         %um, light wavelength
        if nargin < 11
            ff = 0.866;                         %Z7 linear fill factor
            if nargin < 10
                pitch = 4.33;                   %um, pixel pitch for MTF
                if nargin < 9
                    n = 512;                    %< is faster, watch ringing
                    if nargin < 8
                        MTFth = 50;             %lp/mm, MTF50 in-focus threshold
                        if nargin < 7
                            Qeth = 0.8859/2;    %?
                            if nargin < 6
                                QF = 8.8;       %Nikkor 50mm/1.8 S at 10m*
                                if nargin < 5
                                    ns = 20;    %distance samples in decade
                                    u2 = (10.^(1/ns)).^(-30:61); %m
                                    if nargin < 4
                                        plotFlag = true;
                                    end
                                end
                            end
                        end
                    end
                end
            end
        end
    end
    %% Lens LSF and MTF at each distance u2
    Q = 9;                                      %D padding/PSF sampling factor
    wAxis =(-0.5:1/n:0.5-1/n) *2*Q;             %radii, QD in pupil plane
    [U,V] = meshgrid(wAxis);                    %radii, 2D pupil coords
    RHO = sqrt(U.^2 + V.^2);                    %radii, distance from axis
    RHO2 = RHO.^2;                              %for speed
    stop = RHO>1;                               %circular aperture stop
    
    D = fn/N/1000;                              %m, nominal aperture diameter
    fz = 1/(1/u+1000/fn);                       %m, sensing plane distance
    Nz = fz / D;                                %f-number at 'best focus' RMS
    W40 = QF/Nz^2 * lambda;                     %rho^4 Quality Factor
    
    r = (-n/2:n/2-1) / Q * lambda*Nz;           %um, LSF V coords
    f = (0:1/n:1-1/n)' * Q / lambda/Nz;         %cycles per um, V MTF freq.

    W2 = -(sqrt(u^2-D^2/4) - sqrt(u2.^2-D^2/4) - (u - u2)) * 10^6; %um
    
    jk = 1i*2*pi/lambda;
    wf = exp( jk * W40*(RHO.^4-RHO2) );         %fixed, best 'focus'
    lsf = zeros(n,length(u2));                  %object side
    
    for i = 1:length(u2)                        %#ok<*FXUP> For every plane:
        w = wf .* exp(jk .* W2(i) .* RHO2);     %wavefront at exit pupil
        w(stop) = 0;                            %circular aperture
        
        psf = fft2(w);                          %diffracted amplitude
        psf = psf .* conj(psf);                 %intensity = |FT|^2
        
        lsf(:,i) = sum(psf);                    %V LSF at this distance
    end
    
    lsf = [lsf(n/2+1:end,:); lsf(1:n/2,:)];     %fftshift
    lsf = lsf / max(lsf(:));                    %normalized to max intensity
        
    mtfLens = abs(fft(lsf));                    %MTF of lens in V direction
    mtfLens = mtfLens ./ max(mtfLens);          %normalize all to 1 at origin
    
    %% System MTF curve
    mtfPix = abs(sin(pi*f*pitch*ff)./(pi*f*pitch*ff));
    mtfPix(1) = 1;
    mtfSys = mtfLens .* mtfPix;                 %convolve with pixel aperture
    
    % Calculate MTF50
    mtf50 = zeros(1,length(u2));
    for i  = 1:length(u2)                       %MTF50 at each plane
        mtf = mtfSys(:,i);
        I = find(mtf < 0.5, 1);                 %to deal with non monotonicity
        mtf50(i) = interp1(mtf(1:I),f(1:I),0.5);%lp/um
    end
    
    output;
    %--------------------------------------------------------------------
    function output
        %% Calculate Acutance
        %corp.dxomark.com/wp-content/uploads/2017/11/2012-CPIQ_Spacial_Metrics_Copyright.pdf
        d = 400;                                %mm, viewing distance
        dispPitch = 271.8;                      %um, monitor pixel size
        m = dispPitch/pitch;                    %viewing magnification
        
        f_px = f * pitch;                       %c/p pitch
        un = f_px <= 0.5;                       %frequencies to Nyquist
        f_mm = f*1000;                          %c/mm
        %v = f_mm*d/m*pi/180;                   %if tan(theta) = theta
        v = 1./(2*atand(m./(2*d*f_mm)));        %cycles per degree
        
        % Acutance on Y_D65 intensity
        a = 1;                                  %immaterial
        b = 0.2;
        c = 0.8;
        K = 1;                                  %immaterial
        CSFa = a*v.^c.*exp(-b.*v)/K;            %HVS Contrast Sensitivity
        
        Qe = mtfSys .* CSFa;                    %Acutance = HVS response
        Qe = sum(Qe(un,:)) / sum(CSFa);         %du, a and K cancel out
        
        disp(['Plane of ''Best Focus'' fz = ' num2str(fz*1000,5) ' mm, fz/fn = '...
            num2str(1000*fz/fn,3) ', Nz = ' num2str(Nz,4) ', m = ' ...
            num2str(fz/u,3) ])
        disp(['Geometrical Object Focus: ' num2str(u) 'm'])
       
        [M,I,unm,ufm] = dof(mtf50*1000,MTFth);
        disp(['Maximum MTF50           : ' num2str(u2(I)) ' m, at '...
            num2str(M,4) ' lp/mm'])
        
        [M,I,un,uf] = dof(Qe,Qeth);
        disp(['Maximum Edge Acutance   : ' num2str(u2(I)) ' m, Qe = ' num2str(M,3)])
        disp(['Depth of Field (Edge Acutance ' num2str(Qeth) ') = ' ...
            num2str(uf-un,4) ' m: near ' num2str(un,4) ' m, far ' num2str(uf,4) ' m'])
        disp(['Depth of Field (MTF50 ' num2str(MTFth) ' lp/mm) = '...
            num2str(ufm-unm,4) ' m: near ' num2str(unm,4) ' m, far ' num2str(ufm,4) ' m'])
        disp(' ')
 
        if plotFlag
            figure; plot(u2,mtf50*1000); ylim([0 100])
            xlabel('Distance (m)'); ylabel('MTF50 (lp/mm)')
            set(gca,'xscale','log'); grid
            hold on; yyaxis right
            plot(u2,Qe); ylabel('Edge Acutance')
            ylim([0 0.9])   %max discernible difference 0.8859
            
            title([num2str(fn) 'mm lens at ' num2str(u) 'm, f/' num2str(N)...
                ', W040 ' num2str(W40,3) 'um, \lambda ' num2str(lambda) ...
                'um, px effAp ' num2str(pitch*ff,3) 'um'])
        end
        %% --------------------------------------------------------------
        function [M,I,un,uf] = dof(curve,th)
            [M,I] = max(curve);                         %index to CLC (max MTF50)
            un = NaN; uf = NaN;
            try                                         %#ok<*TRYNC>
                un = interp1(curve(1:I),u2(1:I),th); end        %near
            try
                uf = interp1(curve(I:end),u2(I:end),th); end    %far
        end
    end

end